/*
 * Simple commandline argument processing function
 * Programmed and designed by Matti 'ccr' Hamalainen
 * (C) Copyright 2002-2015 Tecnic Software productions (TNSP)
 *
 * Please read file 'COPYING' for information on license and distribution.
 */
#ifndef TH_ARGS
#define TH_ARGS

#include "th_util.h"
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Option flags
 */
#define OPT_NONE             (0)    // Simple option with no arguments
#define OPT_ARGREQ           (1)    // Option requires an argument
#define OPT_ARGMASK          (1)    // Mask for option argument flags

#define OPTH_BAILOUT         0x0001 // Bail out on errors
#define OPTH_ONLY_OPTS       0x0010 // Handle only options
#define OPTH_ONLY_OTHER      0x0020 // Handle only "non-options"
#define OPTH_ONLY_MASK       0x00f0 // Mask


/* Option argument structure
 */
typedef struct
{
    int id;
    char optShort;
    char *optLong;
    char *desc;
    int flags;
} th_optarg_t;


BOOL th_args_process(int argc, char *argv[],
     const th_optarg_t *opts, const int nopts,
     BOOL (*handleOption)(int, char *, char *),
     BOOL (*handleOther)(char *), const int flags);

void th_args_help(FILE *, const th_optarg_t *opts,
     const int nopts, const int flags);

#ifdef __cplusplus
}
#endif
#endif // TH_ARGS
